﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ParalellComputing
{
    // Deklarácia triedy, ktorá zapuzdruje funkcionalitu na realizáciu
    // vybraných grafických transformácií s bitovými mapami.
    class Bitmapa
    {
        // Definície súkromných dátových členov.
        private Bitmap bitovaMapa;
        private string subor;

        // Verejne prístupný parametrický inštančný konštruktor.
        public Bitmapa(string subor)
        {
            bitovaMapa = new Bitmap(subor);
            this.subor = subor;
        }
        // Metóda realizujúca inverziu (tvorbu negatívu) bitovej mapy.
        public void Invertovat()
        {
            int x, y;
            Color farba;
            Bitmap bitmapCopy = (Bitmap)bitovaMapa.Clone();
            for (x = 0; x < bitmapCopy.Width; x++)
            {
                for (y = 0; y < bitmapCopy.Height; y++)
                {
                    farba = bitmapCopy.GetPixel(x, y);
                    bitmapCopy.SetPixel(x, y,
                    Color.FromArgb(255 - farba.R, 255 - farba.G,
                    255 - farba.B));
                }
            }
            FileInfo iSubor = new FileInfo(this.subor);
            string aktualnyPriecinok = iSubor.DirectoryName;
            string novyPriecinok = Directory.CreateDirectory(aktualnyPriecinok +
            "\\Inverzia").FullName;
            bitmapCopy.Save(Path.Combine(novyPriecinok, iSubor.Name));
        }
        // Metóda realizujúca prevod bitovej mapy do sivotónu.
        public void PreviestDoSivotonu()
        {
            int x, y, I;
            Color farba;
            Bitmap bitmapCopy = (Bitmap)bitovaMapa.Clone();
            for (x = 0; x < bitmapCopy.Width; x++)
            {
                for (y = 0; y < bitmapCopy.Height; y++)
                {
                    farba = bitmapCopy.GetPixel(x, y);
                    I = (int)(0.299 * farba.R + 0.587 * farba.G +
                    0.114 * farba.B);
                    bitmapCopy.SetPixel(x, y, Color.FromArgb(I, I, I));
                }
            }
            FileInfo iSubor = new FileInfo(this.subor);
            string aktualnyPriecinok = iSubor.DirectoryName;
            string novyPriecinok = Directory.CreateDirectory(aktualnyPriecinok +
            "\\Sivoton").FullName;
            bitmapCopy.Save(Path.Combine(novyPriecinok, iSubor.Name));
        }
        // Metóda realizujúca prahovanie bitovej mapy.
        public void Prahovat(int prah)
        {
            int x, y, cervena, zelena, modra;
            Color farba;
            Bitmap bitmapCopy = (Bitmap)bitovaMapa.Clone();
            for (x = 0; x < bitmapCopy.Width; x++)
            {
                for (y = 0; y < bitmapCopy.Height; y++)
                {
                    farba = bitmapCopy.GetPixel(x, y);
                    if (farba.R <= prah)
                        cervena = 0;
                    else
                        cervena = 255;
                    if (farba.G <= prah)
                        zelena = 0;
                    else
                        zelena = 255;
                    if (farba.B <= prah)
                        modra = 0;
                    else
                        modra = 255;
                    bitmapCopy.SetPixel(x, y,
                    Color.FromArgb(cervena, zelena, modra));
                }
            }
            FileInfo iSubor = new FileInfo(this.subor);
            string aktualnyPriecinok = iSubor.DirectoryName;
            string novyPriecinok = Directory.CreateDirectory(aktualnyPriecinok +
            "\\Prahovanie").FullName;
            bitmapCopy.Save(Path.Combine(novyPriecinok, iSubor.Name));
        }
        
    }
    class Program
    {
        static void Main(string[] args)
        {
            DirectoryInfo priecinok = new DirectoryInfo(@"..\..\..\Obrazky");
            FileInfo[] subory = priecinok.GetFiles("*.jpg");
            ushort pocetSuborov = (ushort)subory.Length;
            Bitmapa[] bitmapy = new Bitmapa[pocetSuborov];
            Stopwatch stopky = new Stopwatch();
            // Vytvorenie kolekcie bitových máp.
            for (int i = 0; i < pocetSuborov; i++)
            {
                bitmapy[i] = new Bitmapa(subory[i].FullName);
            }
            // Sekvenčné spracovanie grafických transformácií bitových máp.
            Console.WriteLine("Prebiehajú grafické " +
            "transformácie bitových máp.");
            stopky.Start();

            for (int i = 0; i < pocetSuborov; i++)
            {
                bitmapy[i].Invertovat();
                bitmapy[i].PreviestDoSivotonu();
                bitmapy[i].Prahovat(128);
            }

            stopky.Stop();
            Console.WriteLine("Grafické transformácie " +
            "sú hotové [čas: {0} ms].", stopky.ElapsedMilliseconds);
            Console.ReadKey();
        }

    }
}